#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Module d'authentification CAS
"""

import urllib, re, os
from urlparse import urlparse, urljoin
from flask import request, session, redirect, current_app


def parse_tag(chaine, tag):
    """Parse un xml et retourne le tag recherché ou rien
    """
    tag1_pos1 = chaine.find("<" + tag)
    #  No tag found, return empty string.
    if tag1_pos1 == -1:
        return ""
    tag1_pos2 = chaine.find(">", tag1_pos1)
    if tag1_pos2 == -1:
        return ""
    tag2_pos1 = chaine.find("</" + tag, tag1_pos2)
    if tag2_pos1 == -1:
        return ""
    return chaine[tag1_pos2+1:tag2_pos1].strip()

def url_https(url_ini):
    """Retourne url_ini avec https
    """
    if url_ini.startswith('http://'):
        url_ini = url_ini.replace('http','https',1)
    return url_ini
class CASClient:
    """Client CAS
    """
    def __init__(self, active=True):
        """Contructeur
        """
        self.cas_url = current_app.config['CAS_URL']
        self.active = active
        self.active = False
        if not active:
            del(session['netid'])
            del(session['username'])

    def get_id(self):
        """Retourne le session['netid'] ou None
        """
        if session.has_key('netid'):
            return session['netid']
        return None

    def authenticate(self):
        """Authentifie le client auprès du serveur CAS
        """
        if self.is_authenticated():
            return redirect(request.args.get("next"))
        login_url = self.cas_url + 'login' + '?service=' + urllib.quote(url_https(request.url))
        return redirect(login_url)

    def logout(self):
        """Logout du client
        """
        del(session['netid'])
        del(session['username'])
        url = re.sub(r'logout', 'documents', request.url)
        logout_url = self.cas_url + 'logout' + '?service=' + urllib.quote(url)
        return redirect(logout_url)

    def is_authenticated(self):
        """Vérifie si le client est authentifié auprès du serveur CAS
        """
        if session.has_key('netid') and self.active:
            return True
        if request.args.has_key('ticket'):
            username = self.validate(request.args.get('ticket'))
            if username != "":
                session['netid'] = request.args.get('ticket')
                session['username'] = username
                # Création d'un fichier "de session"
                if not os.path.isdir(os.path.join(current_app.config["UPLOAD_FOLDER"], ".sessions")):
                    os.makedirs(os.path.join(current_app.config["UPLOAD_FOLDER"], ".sessions"))
                nomfich = os.path.join(current_app.config["UPLOAD_FOLDER"], ".sessions", session['netid'])
                f = open(nomfich,"w")
                f.close()
                return True
        return False

    def validate(self, ticket):
        """Retourne soit le username en cas de succès soit rien
        """
        cas_validate = self.cas_url + "validate?service=" + urllib.quote(self.serviceURL()) + '&ticket=' + urllib.quote(ticket)
        f_validate   = urllib.urlopen(cas_validate)
        response = f_validate.read()
        if parse_tag(response,"cas:authenticationSuccess") != "":
            return parse_tag(response,"cas:user")
        else:
            return ""

    def serviceURL(self):
        """Retourne une url sans les variables get
        """

        ret = url_https(request.url)
        ret = re.sub(r'ticket=[^&]*&?', '', ret)
        ret = re.sub(r'\?&?$|&$', '', ret)
        return ret
